# -*- coding: utf-8 -*-
import csv
import os
import sys
import re
import binascii
from struct import unpack, pack
import urllib2
import argparse
import hashlib

parser = argparse.ArgumentParser()
parser.add_argument('inputfile', action='store', help='CSV file that contains title information, ideally from: http://yls8.mtheall.com/ninupdates/reports.php')
parser.add_argument('-JPN', action='append_const', dest='regionlist', const='JPN', default=[], help='Download JPN titles')
parser.add_argument('-EUR', action='append_const', dest='regionlist', const='EUR', default=[], help='Download EUR titles')
parser.add_argument('-USA', action='append_const', dest='regionlist', const='USA', default=[], help='Download USA titles')
parser.add_argument('-KOR', action='append_const', dest='regionlist', const='KOR', default=[], help='Download KOR titles')
parser.add_argument('-CHN', action='append_const', dest='regionlist', const='CHN', default=[], help='Download CHN titles')
parser.add_argument('-TWN', action='append_const', dest='regionlist', const='TWN', default=[], help='Download TWN titles')
parser.add_argument('-ALL', action='store_true', default=False, dest='allregions', help='Download all regions')
parser.add_argument('-nosort', action='store_false', default=True, dest='sort', help='Turn OFF sorting of the titles into folders based on their debut fw. For example, if title X was added in FW 4.0.0 and not updated since, by default it is placed in a folder called 4.0.0. So, if you are on 4.0.0 or above, you will not need to install it.')
parser.add_argument('-nobuild', action='store_false', default=True, dest='build', help='Turn OFF generation of CIA files, titles will be downloaded only.')
arguments = parser.parse_args()

execname = './make_cdn_cia'
windoze = '9c4f88f706dedde3bc0ebb66e34963e5'
if hashlib.md5(sys.platform).hexdigest() == windoze:
    execname = 'make_cdn_cia.exe'

def writeInLog(msg):
    f = open("problems.txt", "a")
    f.write(msg+"\n")
    f.close()

##########From https://stackoverflow.com/questions/5783517/downloading-progress-bar-urllib2-python
def chunk_report(bytes_so_far, chunk_size, total_size):
    percent = float(bytes_so_far) / total_size
    percent = round(percent*100, 2)
    sys.stdout.write("Downloaded %d of %d bytes (%0.2f%%)\r" % (bytes_so_far, total_size, percent))
    if bytes_so_far >= total_size:
        sys.stdout.write('\n')

def chunk_read(response, outfname, chunk_size=2*1024*1024, report_hook=None):
    fh = open(outfname,'wb')
    total_size = response.info().getheader('Content-Length').strip()
    total_size = int(total_size)
    bytes_so_far = 0
    data = []
    while 1:
        if report_hook:
            report_hook(bytes_so_far, chunk_size, total_size)
        chunk = response.read(chunk_size)
        bytes_so_far += len(chunk)
        if not chunk:
             break
        fh.write(chunk)
    fh.close()
##########


print '***************************'
print '╦ ╦┌─┐┌┬┐┌─┐┌┬┐┌─┐╔═╗╔╦╗╔╗╔\n║ ║├─┘ ││├─┤ │ ├┤ ║   ║║║║║\n╚═╝┴  ─┴┘┴ ┴ ┴ └─┘╚═╝═╩╝╝╚╝'
print '***************************\n'
print 'UpdateCDN\n'


if not arguments.regionlist and not arguments.allregions:
    print 'No region specified, please choose one or more from: -JPN -EUR -USA -KOR -CHN -TWN or -ALL'
    sys.exit(0)

print '***** DON\'T WORRY about this message: "[!] Caution, Ticket and TMD Title Versions do not match"\n'

f = open(arguments.inputfile)

csvname = "-".join(os.path.splitext(f.name)[0].split())

if not os.path.exists(csvname):
    os.makedirs(csvname)

try:
    reader = csv.reader(f)
    print 'Loaded CSV, now processing...\n'
    #skip first line
    reader.next()
    
    for row in reader:
        
        region = row[1]
        if (region in arguments.regionlist) or arguments.allregions:

            if not os.path.exists(csvname + '/' + region):
                os.makedirs(csvname + '/' + region)

            titleid = row[0]
            version = row[2].split()[-1].strip('v')
            
            if (row[3].split()[-1] == 'scan'):
                debutfw = '2.0.0-2'
            elif (row[3].split()[-1][0] == '('):
                debutfw = row[3].split()[-2] + '-' + row[3].split()[-1].strip('(').strip(')')
            elif (row[3].split()[-1] == 'JPN'):
                debutfw = row[3].split()[-2] + '-' + row[3].split()[-1] + 'after8.1-before9.0'
            else:
                debutfw = row[3].split()[-1]

            if(arguments.sort):
                rawdir = csvname + '/' + region + '/raw/' + debutfw + '/'
                ciadir = csvname + '/' + region + '/cia/' + debutfw +'/'
            else:
                rawdir = csvname + '/' + region + '/raw/'
                ciadir = csvname + '/' + region + '/cia/'

            baseurl = 'http://nus.cdn.c.shop.nintendowifi.net/ccs/download/' + titleid

            try:
                cetk = urllib2.urlopen(baseurl + '/cetk')
            except urllib2.URLError, e:
                print 'ERROR: Could not download CETK?'
                writeInLog('Trouble downloading cetk:  ' + baseurl + '/cetk')
                raise SystemExit(0)
            try:
                tmd = urllib2.urlopen(baseurl + '/tmd.' + version)
            except urllib2.URLError, e:
                print 'ERROR: Could not download TMD?'
                writeInLog('Trouble downloading tmd:  ' + baseurl + '/tmd.' + version)
                raise SystemExit(0)

            tmd = tmd.read()
            cetk = cetk.read()

            if not os.path.exists(rawdir + titleid):
                os.makedirs(rawdir + titleid)

            open(rawdir + titleid + '/cetk','wb').write(cetk)
            open(rawdir + titleid + '/tmd','wb').write(tmd)

            # Download Contents
            contentCount = unpack('>H', tmd[0x206:0x208])[0]
            fSize = 16*1024
            print 'Downloading ' + str(contentCount) + ' title content(s), please wait...'
            for i in xrange(contentCount):
                cOffs = 0xB04+(0x30*i)
                cID = format(unpack('>I', tmd[cOffs:cOffs+4])[0], '08x')
                cIDX = format(unpack('>H', tmd[cOffs+4:cOffs+6])[0], '04x')
                outfname = rawdir + '/' + titleid + '/' + cID
                response = urllib2.urlopen(baseurl + '/' + cID)
                chunk_read(response, outfname, report_hook=chunk_report)
            print 'Title download complete\n'
            
            #cia generation
            if(arguments.build):
                if not os.path.exists(ciadir):
                    os.makedirs(ciadir)
                makecommand = ' ' + rawdir + titleid + ' ' + ciadir + titleid + '.cia'      
                os.system(execname + makecommand)
                print ''            

finally:
    f.close()
    print 'Finished! (If nothing happened/downloaded, check the region you specified exists in the csv you used.)'
